#!/usr/bin/env python
# Copyright (C) 2001 Aaron Lav, asl2@pobox.com

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import urlparse
import urllib
import sys
import readline
import getopt
import string
import mimetools

import slow_output
import ref_extract
import url_completer

# TODO:
#  check on readline.set_word_break?
#  think about integrating labels, urls.
#  think about history

def browse (url, outputter):
    urlfile = urllib.urlopen (url)
    msg = urlfile.info() # mimetypes.guess_type is my friend for non-http
    if (msg.type == "text/html"):
	p = ref_extract.RefExtractHTMLParser()
    else:
	p = ref_extract.NullParser ()
	
    while 1:
	# if urlfile.readlines () were a lazy sequence, or if I were willing
	# to wait until all data had been read before displaying any, this
	# loop could be more modular.
	data = urlfile.readline ()
	if data == "":
	    break
	outputter.write (data)
	p.feed (data)
    p.close ()

    # XXX should only extract refs and canonicalize if really html
    
    # XXX is there a functional difference between urlparse.urljoin
    # and urllib.basejoin?  Neither seem to handle .. relative URLs.
    
    urllist = map (
	(lambda partial, baseurl = url: urlparse.urljoin (baseurl,partial)),
	p.anchorlist)

    url_completer.URLCompleter(urllist)

name_to_speed = {"V.21":300,
		 "V.22":1200,
		 "V.22BIS":2400,
		 "103":300,
		 "212":1200,
                 "V.32BIS":14400,
		 "DSL":1e6} # more convenience than accuracy ...

if __name__ == '__main__':
    readline.parse_and_bind("tab: complete")
    speed = 2400
    (opts, args) = getopt.getopt (sys.argv[1:], 'm:') 
    for (opt, arg) in opts:
	if opt == "-m":
	    try:
		speed = name_to_speed [string.upper(arg)]
	    except KeyError:
		print "Bad modulation name", arg, "defaulting to", speed
    outputter = slow_output.slow_output ()
    outputter.set_delay_secs(10.0/speed)
    # 1 start bit,8 data,1 stop -> 10 bits/char

    for arg in range(0,len(args)):
	browse(args[arg],outputter)
    while 1:
	try:
	    tmp = raw_input ()
	except EOFError:
	    break
	if not tmp: break
	browse (tmp,outputter)



